using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Data;

namespace Vevo.Data
{
    public class ConfigurationRepositoryNoCache : IConfigurationRepository
    {
        #region Private

        ConfigurationValueAccess _configurationValueAccess = new ConfigurationValueAccess();
        ConfigurationDescriptionAccess _configurationDescriptionAccess = new ConfigurationDescriptionAccess();

        private const string AllConfigFieldsWithConfigDescription =
            " Configuration.ConfigID, Name, Searchable, CultureID, MainGroup, SubGroup, Description, Keyword, " +
            " DisplayName, ConfigType, SelectionValues, SelectionNames, SelectionImages, ParentConfigName, " +
            " DisplayUserControl, EnablerConfigName, FieldValidation ";

        private const string AllConfigFields =
            " ConfigID, Name, Searchable, ConfigType, SelectionValues, SelectionNames, " +
            " SelectionImages, ParentConfigName, DisplayUserControl, EnablerConfigName, FieldValidation ";

        private Configuration.ConfigurationType ConfigurationType( string type )
        {
            if (String.IsNullOrEmpty( type ))
                return Configuration.ConfigurationType.Unknow;
            else
                return EnumUtilities<Configuration.ConfigurationType>.Parse( type );
        }

        private Configuration CreateOneConfigurationFromRow( DataRow row )
        {
            Configuration config = new Configuration( row["Name"].ToString() );
            config.ConfigID = row["ConfigID"].ToString();
            config.Searchable = ConvertUtilities.ToBoolean( row["Searchable"] );
            config.ConfigType = ConfigurationType( row["ConfigType"].ToString() );
            config.SelectionValues = row["SelectionValues"].ToString();
            config.SelectionNames = row["SelectionNames"].ToString();
            config.SelectionImages = row["SelectionImages"].ToString();
            config.ParentConfigName = row["ParentConfigName"].ToString();
            config.DisplayUserControl = row["DisplayUserControl"].ToString();
            config.EnablerConfigName = row["EnablerConfigName"].ToString();
            config.FieldValidation = row["FieldValidation"].ToString();
            return config;
        }
        private ConfigurationValue CreateOneValueFromRow( DataRow row )
        {
            ConfigurationValue value = new ConfigurationValue( row["CultureID"].ToString(), row["StoreID"].ToString() );
            value.ItemValue = row["ItemValue"].ToString();

            return value;
        }

        private void AddValues( Configuration config, DataTable configValueTable )
        {
            DataRow[] configValueRows =
                configValueTable.Select( "ConfigID = " + config.ConfigID );
            foreach (DataRow row in configValueRows)
                config.Values.Add( CreateOneValueFromRow( row ) );

            // Mark this list as unchanged. Otherwise, Save() funciton will think that added values are new
            config.Values.AcceptChanges();
        }

        private ConfigurationDescription CreateOneDescriptionFromRow( DataRow row )
        {
            ConfigurationDescription description = new ConfigurationDescription( row["CultureID"].ToString() );
            description.MainGroup = row["MainGroup"].ToString();
            description.SubGroup = row["SubGroup"].ToString();
            description.Description = row["Description"].ToString();
            description.Keyword = row["Keyword"].ToString();
            description.DisplayName = row["DisplayName"].ToString();

            return description;
        }

        private void AddDescriptions( Configuration config, DataTable configDescriptionTable )
        {
            DataRow[] configDescriptionRows =
                configDescriptionTable.Select( "ConfigID = " + config.ConfigID );

            foreach (DataRow row in configDescriptionRows)
                config.Descriptions.Add( CreateOneDescriptionFromRow( row ) );

            // Mark this list as unchanged. Otherwise, Save() funciton will think that added values are new
            config.Descriptions.AcceptChanges();
        }

        private IList<Configuration> ConvertConfigurationTableToList(
            DataTable configTable,
            DataTable configValueTable,
            DataTable configDescriptionTable )
        {
            IList<Configuration> list = new List<Configuration>();

            foreach (DataRow row in configTable.Rows)
            {
                Configuration config = CreateOneConfigurationFromRow( row );
                AddValues( config, configValueTable );
                AddDescriptions( config, configDescriptionTable );

                list.Add( config );
            }

            return list;
        }

        private IList<Configuration> ConvertConfigurationTableToList(
            Culture culture, DataTable table )
        {
            DataTable configValueTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, ItemValue, StoreID " +
                "FROM ConfigurationValue; " );

            IList<Configuration> list = new List<Configuration>();
            foreach (DataRow row in table.Rows)
            {
                Configuration config = CreateOneConfigurationFromRow( row );
                AddValues( config, configValueTable );
                AddDescriptions( config, table );
                list.Add( config );
            }
            return list;

        }

        private void CreateConfig( Configuration configuration )
        {
            string configID = DataAccess.ExecuteScalar(
                "INSERT INTO Configuration (Name, Searchable, ConfigType, SelectionValues, SelectionNames, " +
                "SelectionImages, ParentConfigName, DisplayUserControl, EnablerConfigName, FieldValidation ) " +
                "VALUES (@Name, @Searchable, @ConfigType, @SelectionValues, @SelectionNames, " +
                "@SelectionImages, @ParentConfigName, @DisplayUserControl, @EnablerConfigName, " +
                "@FieldValidation ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( configuration.Name ),
                DataAccess.CreateParameterBool( configuration.Searchable ),
                DataAccess.CreateParameterString( configuration.ConfigType.ToString() ),
                DataAccess.CreateParameterString( configuration.SelectionValues ),
                DataAccess.CreateParameterString( configuration.SelectionNames ),
                DataAccess.CreateParameterString( configuration.SelectionImages ),
                DataAccess.CreateParameterString( configuration.ParentConfigName ),
                DataAccess.CreateParameterString( configuration.DisplayUserControl ),
                DataAccess.CreateParameterString( configuration.EnablerConfigName ),
                DataAccess.CreateParameterString( configuration.FieldValidation )
                );

            configuration.ConfigID = configID;
        }

        private void UpdateConfig( Configuration configuration )
        {
            string script = "UPDATE Configuration SET Searchable = @Searchable, ConfigType = @ConfigType, " +
                "SelectionValues = @SelectionValues, SelectionNames = @SelectionNames,  " +
                "SelectionImages = @SelectionImages, ParentConfigName = @ParentConfigName, " +
                "DisplayUserControl = @DisplayUserControl, EnablerConfigName = @EnablerConfigName, " +
                "FieldValidation = @FieldValidation WHERE Name = @Name; ";

            DataAccess.ExecuteNonQuery( script,
                DataAccess.CreateParameterBool( configuration.Searchable ),
                DataAccess.CreateParameterString( configuration.ConfigType.ToString() ),
                DataAccess.CreateParameterString( configuration.SelectionValues ),
                DataAccess.CreateParameterString( configuration.SelectionNames ),
                DataAccess.CreateParameterString( configuration.SelectionImages ),
                DataAccess.CreateParameterString( configuration.ParentConfigName ),
                DataAccess.CreateParameterString( configuration.DisplayUserControl ),
                DataAccess.CreateParameterString( configuration.EnablerConfigName ),
                DataAccess.CreateParameterString( configuration.FieldValidation ),
                DataAccess.CreateParameterString( configuration.Name )
                );

        }

        private Configuration Create( Configuration configuration )
        {
            CreateConfig( configuration );
            _configurationValueAccess.SaveAll( configuration.ConfigID, configuration.Values );
            _configurationDescriptionAccess.SaveAll( configuration.ConfigID, configuration.Descriptions );

            return configuration;
        }

        private Configuration Update( Configuration configuration )
        {
            UpdateConfig( configuration );
            _configurationValueAccess.SaveAll( configuration.ConfigID, configuration.Values );
            _configurationDescriptionAccess.SaveAll( configuration.ConfigID, configuration.Descriptions );

            return configuration;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.ToLower().StartsWith( "configid" ))
                return "ORDER BY Configuration." + sortBy;
            else
                return "ORDER BY " + sortBy;
        }

        private string ConditionSearchField(
            string searchTerm,
            string[] searchFieldList,
            ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( searchTerm ))
            {
                string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ', '@', ';', '\'' } );
                string query = "";
                for (int i = 0; i < arKeys.Length; i++)
                {
                    if (i == 0)
                    {
                        query = " AND (";
                    }
                    for (int j = 0; j < searchFieldList.Length; j++)
                    {
                        string strColumn = searchFieldList[j];
                        if (j == 0 && i == 0)
                            query += " ";
                        else
                            query += " OR ";

                        query += strColumn;

                        parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[i] + "%" ) );
                        query += " LIKE @" + strColumn + i;
                    }
                    if (i == arKeys.Length - 1)
                        query += " ) ";
                }
                return query;
            }
            else
                return "";
        }

        #endregion


        #region Public Methods

        public Configuration Save( Configuration configuration )
        {
            Configuration result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (configuration.ConfigID == "0")
                    result = Create( configuration );
                else
                    result = Update( configuration );

                transaction.Complete();
            }

            return result;
        }

        // Update values for existing items only. No change in descriptions and other fields
        public Configuration UpdateValue( Configuration configuration, string newValue )
        {
            return UpdateValue( configuration, newValue, Store.Null );
        }

        public Configuration UpdateValue( Configuration configuration, string newValue, Store store )
        {
            if (!configuration.CanUseSingleCulture())
                throw new DataException(
                   "The configuration '" + configuration.Name
                   + "' supports multi-culture and requires culture parameter." );

            _configurationValueAccess.Save( configuration.ConfigID, newValue, store.StoreID );
            configuration.SetValue( newValue, store );

            return configuration;
        }

        // Update values for existing items only. No change in descriptions and other fields
        public Configuration UpdateValue( Culture culture, Configuration configuration, string newValue )
        {
            return UpdateValue( culture, configuration, newValue, Store.Null );
        }

        public Configuration UpdateValue(
            Culture culture, Configuration configuration, string newValue, Store store )
        {
            if (culture.CultureID != Culture.Null.CultureID
                && !configuration.CanUseMultiCulture())
                throw new DataException(
                    "The configuration '" + configuration.Name + "' does not support multi-culture " +
                    "and does not accept culture parameter. " );

            _configurationValueAccess.Save( culture.CultureID, configuration.ConfigID, newValue, store.StoreID );
            configuration.SetValue( culture.CultureID, newValue, store.StoreID );

            return configuration;
        }

        public IList<Configuration> GetAll()
        {
            DataTable configTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, Name, Searchable, ConfigType, SelectionValues, SelectionNames, " +
                "SelectionImages, ParentConfigName, DisplayUserControl, EnablerConfigName, FieldValidation " +
                "FROM Configuration; " );

            DataTable configValueTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, ItemValue, StoreID " +
                "FROM ConfigurationValue; " );

            DataTable configDescriptionTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, MainGroup, SubGroup, Description, Keyword, DisplayName " +
                "FROM ConfigurationDescription; " );

            return ConvertConfigurationTableToList( configTable, configValueTable, configDescriptionTable );
        }

        public IList<Configuration> SearchConfiguration(
            Culture culture, string sortBy, string searchTerm, string[] searchFieldList )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            string script = "SELECT " + AllConfigFieldsWithConfigDescription +
                "FROM Configuration LEFT JOIN " +
                "     (SELECT * FROM ConfigurationDescription WHERE CultureID = @CultureID) AS L " +
                "     ON Configuration.ConfigID = L.ConfigID " +
                "     WHERE  Searchable = @Searchable " +
                ConditionSearchField( searchTerm, searchFieldList, parameterList ) +
                CreateSortByText( sortBy ) + "; ";

            DataTable full = DataAccess.ExecuteSelect(
                script, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return ConvertConfigurationTableToList( culture, full );
        }

        public IList<Configuration> GetByParentConfigName( string configName )
        {
            string script = "SELECT " + AllConfigFields +
                "FROM Configuration " +
                "WHERE  ParentConfigName = @ParentConfigName ;";

            DataTable table = DataAccess.ExecuteSelect( script,
                DataAccess.CreateParameterString( configName ) );

            DataTable configValueTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, ItemValue, StoreID " +
                "FROM ConfigurationValue; " );

            DataTable configDescriptionTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, MainGroup, SubGroup, Description, Keyword, DisplayName " +
                "FROM ConfigurationDescription; " );

            return ConvertConfigurationTableToList( table, configValueTable, configDescriptionTable );
        }


        public IList<Configuration> GetByEnablerConfigName( string configName )
        {
            string script = "SELECT " + AllConfigFields +
                "FROM Configuration " +
                "WHERE  EnablerConfigName = @EnablerConfigName ;";

            DataTable table = DataAccess.ExecuteSelect( script,
                DataAccess.CreateParameterString( configName ) );

            DataTable configValueTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, ItemValue, StoreID " +
                "FROM ConfigurationValue; " );

            DataTable configDescriptionTable = DataAccess.ExecuteSelect(
                "SELECT ConfigID, CultureID, MainGroup, SubGroup, Description, Keyword, DisplayName " +
                "FROM ConfigurationDescription; " );

            return ConvertConfigurationTableToList( table, configValueTable, configDescriptionTable );
        }

        public void DeleteAll()
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM Configuration; " );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ConfigurationValue; " );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ConfigurationDescription; " );
        }

        public void DeleteConfigValueByStoreID( string storeID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ConfigurationValue " +
                "WHERE StoreID = @StoreID ; ",
                DataAccess.CreateParameterString( storeID ) );
        }

        #endregion
    }
}
