﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.GoogleFeed;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GoogleFeedTagRepository : IGoogleFeedTagRepository
    {
        private const string RepositoryName = "GoogleFeedTagRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetAllSignature = "GetAll";
        private const string GetAllByTagTypeSignature = "GetAllByTagType";
        private const string SearchGoogleFeedTagSignature = "SearchGoogleFeedTag";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "GoogleFeedTag", "GoogleFeedTagType" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private GoogleFeedTagRepositoryNoCache _repository = new GoogleFeedTagRepositoryNoCache();

        #region Constructors

        public GoogleFeedTagRepository()
        {
        }

        public GoogleFeedTagRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region GoogleFeedTagRepository Members

        public GoogleFeedTag Save( GoogleFeedTag googleFeedTag )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            GoogleFeedTag newGoogleFeedTag = _repository.Save( googleFeedTag );
            _dataCache.ClearRepositoryCache();
            return newGoogleFeedTag;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public GoogleFeedTag GetOne( string googleFeedTagID )
        {
            GoogleFeedTag result = (GoogleFeedTag) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, googleFeedTagID );

            if (result == null)
            {
                result = _repository.GetOne( googleFeedTagID );
                _dataCache.SetValue( GetOneSignature, result, googleFeedTagID );
            }

            return result;
        }

        public IList<GoogleFeedTag> GetAll()
        {
            IList<GoogleFeedTag> result = (IList<GoogleFeedTag>) _dataCache.GetValue(
                 GetAllSignature, _cacheDependency );

            if (result == null)
            {
                result = new List<GoogleFeedTag>();
                result = _repository.GetAll();
                _dataCache.SetValue(
                    GetAllSignature,
                    result
                    );
            }
            return result;
        }

        public IList<GoogleFeedTag> GetAll( string sortBy )
        {
            IList<GoogleFeedTag> result = (IList<GoogleFeedTag>) _dataCache.GetValue(
                 GetAllSignature, _cacheDependency, sortBy );

            if (result == null)
            {
                result = new List<GoogleFeedTag>();
                result = _repository.GetAll();
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    sortBy
                    );
            }
            return result;
        }

        public IList<GoogleFeedTag> GetAllByTagTypeID( string googleFeedTagTypeID )
        {
            IList<GoogleFeedTag> result = (IList<GoogleFeedTag>) _dataCache.GetValue(
                 GetAllByTagTypeSignature, _cacheDependency, googleFeedTagTypeID );

            if (result == null)
            {
                result = new List<GoogleFeedTag>();
                result = _repository.GetAllByTagTypeID( googleFeedTagTypeID );
                _dataCache.SetValue(
                    GetAllByTagTypeSignature,
                    result,
                    googleFeedTagTypeID
                    );
            }
            return result;
        }

        public IList<GoogleFeedTag> GetAllByTagTypeID( string sortBy, string googleFeedTagTypeID )
        {
            IList<GoogleFeedTag> result = (IList<GoogleFeedTag>) _dataCache.GetValue(
                 GetAllByTagTypeSignature, _cacheDependency, sortBy, googleFeedTagTypeID );

            if (result == null)
            {
                result = new List<GoogleFeedTag>();
                result = _repository.GetAllByTagTypeID( sortBy, googleFeedTagTypeID );
                _dataCache.SetValue(
                    GetAllByTagTypeSignature,
                    result,
                    sortBy,
                    googleFeedTagTypeID
                    );
            }
            return result;
        }

        public void Delete( string googleFeedTagID )
        {
            _repository.Delete( googleFeedTagID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<GoogleFeedTag> SearchGoogleFeedTag(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchGoogleFeedTagSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchGoogleFeedTag( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchGoogleFeedTagSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<GoogleFeedTag>) result.First;
        }

        #endregion
    }
}
