using System;
using System.Collections;
using System.Collections.Generic;

namespace Vevo.QuickBooks
{
    /// <summary>
    /// Summary description for QBWCSessionPool
    /// </summary>
    public static class QBWCSessionPool
    {
        private const int VerifyExpiredSessionIntervalSeconds = 300;        // 5 minutes

        private static Dictionary<string, QBWCSession> _sessions;
        
        private static int _idleTimeoutSeconds;
        private static DateTime _lastVerifyExpiredSessionTime;

        private static void ResetVerifyExpiredSessionTime()
        {
            _lastVerifyExpiredSessionTime = DateTime.MinValue;
        }

        private static void UpdateVerifyExpiredSessionTime()
        {
            _lastVerifyExpiredSessionTime = DateTime.Now;
        }

        private static bool IsTimeToClear()
        {
            TimeSpan timePassed = DateTime.Now.Subtract( _lastVerifyExpiredSessionTime );

            return timePassed >= new TimeSpan( 0, 0, VerifyExpiredSessionIntervalSeconds );
        }

        private static void ClearExpiredSession()
        {
            if (!IsTimeToClear())
                return;

            UpdateVerifyExpiredSessionTime();

            List<string> expiredSessions = new List<string>();

            IDictionaryEnumerator enumerator = _sessions.GetEnumerator();
            while (enumerator.MoveNext())
            {
                QBWCSession session = (QBWCSession) enumerator.Value;
                if (session.IsExpired( IdleTimeoutSeconds ))
                    expiredSessions.Add( session.Ticket );
            }

            foreach (string ticket in expiredSessions)
            {
                _sessions.Remove( ticket );
            }
        }

        static QBWCSessionPool()
        {
            _sessions = new Dictionary<string, QBWCSession>();
            _idleTimeoutSeconds = DefaultIdleTimeoutSeconds;

            ResetVerifyExpiredSessionTime();
        }

        public const int DefaultIdleTimeoutSeconds = 60 * 60;      // 1 hour

        public static int IdleTimeoutSeconds
        {
            get
            {
                return _idleTimeoutSeconds;
            }
            set
            {
                ResetVerifyExpiredSessionTime();

                _idleTimeoutSeconds = value;
            }
        }

        public static string CreateSession( string qbLogID )
        {
            QBWCSession session = new QBWCSession();
            _sessions.Add( session.Ticket, session );
            session.QBLogID = qbLogID;
            return session.Ticket;
        }

        public static QBWCSession GetSession( string ticket )
        {
            ClearExpiredSession();
            string key = ticket.ToLower();

            if (!_sessions.ContainsKey( key ))
                return null;

            return _sessions[key];
        }

        public static void DeleteSession( string ticket )
        {
            if (_sessions.ContainsKey( ticket ))
                _sessions.Remove( ticket );
        }
    }
}
